# -*- coding: utf-8 -*-
"""
Created on Sat Oct  3 21:45:27 2020

@author: win
"""

import xlrd
import os
from .dictionary import Dictionary
import re
import nltk
import openpyxl
from .log_wp import Log_wp

class Table_extraction:
    def __init__(self, excels_path, C_path, prop_name='solvus'):
        self.excels_path = excels_path
        self.C_path = C_path
        self.prop_name = prop_name
        self.dict_info = Dictionary(self.C_path)
        self.ele_list = self.dict_info.ele_list
        self.e_pattern = self.dict_info.table_e_pattern
        self.ratio_pattern = self.dict_info.table_ratio_pattern
        self.prop_pattern = self.dict_info.table_prop_pattern
        self.unit_pattern = self.dict_info.unit_pattern_table
        self.number_pattern = self.dict_info.table_number_pattern
        self.ele_to_abr = self.dict_info.ele_to_abr
        self.prop_pattern_words = self.dict_info.table_prop_pattern_words
        self.log_wp = Log_wp()

    def composition_triple_extraction(self):
        file_list = os.listdir(self.excels_path)
        composition_all = {}
        for excel_path in file_list:
            try:
                file = xlrd.open_workbook(self.excels_path + '/' + excel_path)
                all_material = []               
                for sheet_i in range(len(file.sheets())):
                    try:
                        sheet = file.sheet_by_index(sheet_i)
                        topic = sheet.row_values(1)[0] 
                        if 'composition' in topic.lower():
                            target_ele_row = []
                            target_ele_col = []
                            search_outcome = []
                            ele_loc = None
                            for line_index in range(2,len(sheet.col_values(0))):
                                search_line = sheet.row_values(line_index)
                                unit_i = 0
                                for unit in search_line:
                                    outcome = re.findall(self.e_pattern,str(unit))
                                    if outcome and str(unit) in self.ele_list:
                                        target_ele_row.append(line_index)
                                        target_ele_col.append(unit_i)
                                        search_outcome.append(unit)
                                    unit_i += 1  
                                if search_outcome:
                                    ele_loc = line_index
                                    break
                            if ele_loc:
                                first_col = sheet.col_values(0)
                                dict_info = Dictionary(self.C_path)
                                alloy_replace = dict_info.table_alloy_to_replace
                                alloy_common_type = dict_info.alloy_writing_type
                                alloy_blank_type = dict_info.alloy_blank_type
                                for alloy_model,replace in alloy_replace.items():
                                    alloy_part = re.findall(alloy_model,str(topic))
                                    for alloy in alloy_part:
                                        find_part = re.findall(replace[0],str(alloy))
                                        alloy_out = alloy.replace(find_part[0],replace[1])
                                        topic = topic.replace(alloy,alloy_out)
                                    outcome_name = []
                                    topic_tokenize = nltk.word_tokenize(topic)
                                    for word in topic_tokenize:
                                        for pattern_1 in alloy_common_type:
                                            outcome_common = re.findall(pattern_1,str(word))
                                            if outcome_common:
                                                outcome_name.append(word)
                                                break        
                                    for pattern_2 in alloy_blank_type:
                                        outcome_blank = re.findall(pattern_2,str(topic))
                                        if outcome_blank and outcome_blank[0] not in outcome_name:
                                            outcome_name.append(outcome_blank[0])
                                            break
                                len_col = len(sheet.row_values(3))
                                alloy_name_col = None
                                alloy_name_search = []
                                if len_col <= 3:
                                    for col_i in range(len_col):
                                        col_info = sheet.col_values(col_i)
                                        if col_i == 0:
                                            col_info = sheet.col_values(col_i)[2:]
                                        if col_info:
                                            for cell in col_info:
                                                for pattern_1 in alloy_common_type:
                                                    outcome_common = re.findall(pattern_1,str(cell))
                                                    if outcome_common:
                                                        alloy_name_col = col_i
                                                        alloy_name_search.append(col_i)
                                                for pattern_2 in alloy_blank_type:
                                                    outcome_blank = re.findall(pattern_2,str(cell))
                                                    if outcome_blank:
                                                        alloy_name_col = col_i
                                                        alloy_name_search.append(col_i)
                                else:
                                    for col_i in range(3):
                                        col_info = sheet.col_values(col_i)
                                        if col_i == 0:
                                            col_info = sheet.col_values(col_i)[2:]
                                        if col_info:
                                            for cell in col_info:
                                                for pattern_1 in alloy_common_type:
                                                    outcome_common = re.findall(pattern_1,str(cell))
                                                    if outcome_common:
                                                        alloy_name_col = col_i
                                                        alloy_name_search.append(col_i)
                                                for pattern_2 in alloy_blank_type:
                                                    outcome_blank = re.findall(pattern_2,str(cell))
                                                    if outcome_blank:
                                                        alloy_name_col = col_i
                                                        alloy_name_search.append(col_i)
                                if not alloy_name_search:
                                    alloy_name_col = 0
                                else:
                                    alloy_name_col = alloy_name_search[0]
                                first_col = sheet.col_values(0)
                                ele_first = []
                                for unit in first_col:
                                    firstcol_search = re.findall(self.e_pattern,str(unit))
                                    if firstcol_search:
                                        ele_first.append(unit)
                                if len(ele_first) <= 2:
                                    if len(first_col) > 4:
                                        e_search = re.findall(self.e_pattern,str(sheet.col_values(0)[ele_loc]))
                                        if e_search and outcome_name and len(outcome_name) == 1:
                                            for index_row in range(ele_loc+1,len(first_col)):
                                                composition_single = {}
                                                composition_single['material'] = outcome_name[0].replace('~',' ')
                                                composition_single['doi']=first_col[0]
                                                ratio_find_topic = re.findall(self.ratio_pattern,str(topic))
                                                ratio_find_col = re.findall(self.ratio_pattern,str(first_col[index_row]))
                                                for table_head in sheet.row_values(2):
                                                    ratio_find_head = re.findall(self.ratio_pattern,str(table_head))
                                                    if ratio_find_head:
                                                        composition_single['percentage'] = ratio_find_head[0]
                                                        break
                                                if ratio_find_topic:
                                                    composition_single['percentage'] = ratio_find_topic[0]
                                                elif ratio_find_col:
                                                    composition_single['percentage'] = ratio_find_col[0]   
                                                for ele_index in range(len(sheet.row_values(2))):
                                                    ele_name = sheet.row_values(ele_loc)[ele_index]
                                                    if ele_name in tuple(self.ele_to_abr.keys()):
                                                        ele_name = self.ele_to_abr[ele_name]
                                                    number = sheet.row_values(index_row)[ele_index]
                                                    composition_single[ele_name]=number
                                                all_material.append(composition_single)
                                        if not e_search:
                                            for index_row in range(ele_loc+1,len(first_col)):
                                                if first_col[index_row]:
                                                    composition_single = {}
                                                    name_col = sheet.col_values(alloy_name_col)
                                                    if outcome_name and len(outcome_name) == 1 and not alloy_name_search:
                                                        composition_single['material'] = outcome_name[0].replace('~', ' ')
                                                    else:
                                                        composition_single['material'] = name_col[index_row]
                                                    composition_single['doi']=first_col[0]
                                                    ratio_find_topic = re.findall(self.ratio_pattern,str(topic))
                                                    ratio_find_col = re.findall(self.ratio_pattern,str(first_col[index_row]))
                                                    for table_head in sheet.row_values(2):
                                                        ratio_find_head = re.findall(self.ratio_pattern,str(table_head))
                                                        if ratio_find_head:
                                                            composition_single['percentage'] = ratio_find_head[0]
                                                            break
                                                    if ratio_find_topic:
                                                        composition_single['percentage'] = ratio_find_topic[0]
                                                    elif ratio_find_col:
                                                        composition_single['percentage'] = ratio_find_col[0]
                                                    ratio_find_unit = re.findall(self.ratio_pattern,str(first_col[index_row]))
                                                    if ratio_find_unit:
                                                        composition_single['percentage'] = ratio_find_unit[0]
                                                    for ele_index in range(len(sheet.row_values(ele_loc)[1:])):
                                                        ele_name = sheet.row_values(ele_loc)[1:][ele_index]
                                                        if ele_name in tuple(self.ele_to_abr.keys()):
                                                            ele_name = self.ele_to_abr[ele_name]
                                                        number = sheet.row_values(index_row)[ele_index+1]
                                                        composition_single[ele_name]=number
                                                    all_material.append(composition_single)
                                    else:
                                        composition_single = {}
                                        first_col_1 = sheet.row_values(3)[0]
                                        e_search = re.findall(self.e_pattern,str(sheet.col_values(0)[ele_loc]))
                                        ratio_find_col = re.findall(self.ratio_pattern,str(first_col_1))
                                        for table_head in sheet.row_values(2):
                                            ratio_find_head = re.findall(self.ratio_pattern,str(table_head))
                                            if ratio_find_head:
                                                composition_single['percentage'] = ratio_find_head[0]
                                                break
                                        if ratio_find_col:
                                            composition_single['percentage'] = ratio_find_col[0]
                                        ratio_find_topic = re.findall(self.ratio_pattern,str(topic))
                                        if ratio_find_topic:
                                            composition_single['percentage'] = ratio_find_topic[0]
                                        if outcome_name and e_search:
                                            composition_single['material'] = outcome_name[0].replace('~',' ')
                                            composition_single['doi']=first_col[0]
                                            for ele_index in range(len(sheet.row_values(2))):
                                                ele_name = sheet.row_values(ele_loc)[ele_index]
                                                number = sheet.row_values(3)[ele_index]
                                                if ele_name in tuple(self.ele_to_abr.keys()):
                                                    ele_name = self.ele_to_abr[ele_name]
                                                composition_single[ele_name]=number
                                            all_material.append(composition_single)
                                        elif outcome_name and not e_search:
                                            if len(outcome_name) == 1:
                                                composition_single['material'] = outcome_name[0].replace('~',' ')
                                            else:
                                                composition_single['material'] = sheet.row_values(ele_loc+1)[alloy_name_col]
                                            composition_single['doi']=first_col[0]
                                            for ele_index in range(len(sheet.row_values(2)[1:])):
                                                ele_name = sheet.row_values(ele_loc)[1:][ele_index]
                                                number = sheet.row_values(3)[1:][ele_index]
                                                if ele_name in tuple(self.ele_to_abr.keys()):
                                                    ele_name = self.ele_to_abr[ele_name]
                                                composition_single[ele_name]=number
                                            all_material.append(composition_single)
                                        elif not outcome_name and not e_search:
                                            composition_single['material'] = sheet.row_values(ele_loc+1)[alloy_name_col]
                                            composition_single['doi']=first_col[0]
                                            m_name = sheet.row_values(ele_loc)[0]
                                            composition_single[m_name] = first_col[3]
                                            for ele_index in range(len(sheet.row_values(2)[1:])):
                                                ele_name = sheet.row_values(ele_loc)[1:][ele_index]
                                                number = sheet.row_values(3)[1:][ele_index]
                                                if ele_name in tuple(self.ele_to_abr.keys()):
                                                    ele_name = self.ele_to_abr[ele_name]
                                                composition_single[ele_name]=number
                                            all_material.append(composition_single)                                                                                                          
                                        elif not outcome_name and e_search:
                                            composition_single['material'] = None
                                            composition_single['doi']=first_col[0]
                                            for ele_index in range(len(sheet.row_values(2))):
                                                ele_name = sheet.row_values(ele_loc)[ele_index]
                                                number = sheet.row_values(3)[ele_index]
                                                if ele_name in tuple(self.ele_to_abr.keys()):
                                                    ele_name = self.ele_to_abr[ele_name]
                                                composition_single[ele_name]=number
                                            all_material.append(composition_single)
                                else:
                                    ele_row = sheet.row_values(ele_loc-1)
                                    len_elerow = len(ele_row)
                                    for index_col in range(1,len_elerow):
                                        if ele_row[index_col] :
                                            composition_single = {}
                                            if outcome_name and len(outcome_name) == 1 and len_elerow <=2:
                                                material_name = outcome_name[0].replace('~',' ')
                                            else:
                                                material_name = ele_row[index_col]
                                            composition_single['material']=material_name
                                            composition_single['doi']=first_col[0]
                                            ratio_find_topic = re.findall(self.ratio_pattern,str(topic))
                                            ratio_find_col = re.findall(self.ratio_pattern,str(material_name))
                                            if ratio_find_topic:
                                                composition_single['percentage'] = ratio_find_topic[0]
                                            elif ratio_find_col:
                                                composition_single['percentage'] = ratio_find_col[0]
                                            for ele_index in range(len(sheet.col_values(0)[ele_loc:])):
                                                ele_name = sheet.col_values(0)[ele_loc:][ele_index]
                                                number = sheet.col_values(index_col)[ele_loc+ele_index]
                                                if ele_name in tuple(self.ele_to_abr.keys()):
                                                    ele_name = self.ele_to_abr[ele_name]
                                                composition_single[ele_name]=number
                                            all_material.append(composition_single)
                        if all_material:
                            break
                    except Exception as e:
                        self.log_wp.print_log(str(e))
                        self.log_wp.print_log("An error in the %s of %s!",sheet_i,excel_path)
                if all_material:
                    composition_all[excel_path] = all_material
            except Exception as e:
                self.log_wp.print_log("can't open this file, name of file is %s", str(excel_path))
                self.log_wp.print_log("Error is %s", str(e))
                self.log_wp.print_log("--"*25)
        return composition_all

    def property_info_extraction(self):
        file_list = os.listdir(self.excels_path)
        property_all = {}
        number_prop = 0
        K_path = []
        for excel_path in file_list:
           try:
                file = xlrd.open_workbook(self.excels_path + '/' + excel_path)
                all_material = []
                for sheet_i in range(len(file.sheets())):
                   try:
                        sheet = file.sheet_by_index(sheet_i)
                        topic = sheet.row_values(1)[0]
                        search_outcome = []
                        target_prop_row = []
                        target_prop_col = []
                        for line_index in range(2,len(sheet.col_values(0))):
                            search_line = sheet.row_values(line_index)[1:]
                            unit_i = 1
                            for unit in search_line:
                                outcome_words = None
                                for pattern in self.prop_pattern[self.prop_name]:
                                    outcome = re.findall(pattern,str(unit))
                                    if all(word in str(unit) for word in self.prop_pattern_words[self.prop_name]):
                                        outcome_words = unit
                                    if outcome:
                                        break
                                if outcome or outcome_words:
                                    target_prop_row.append(line_index)
                                    
                                    target_prop_col.append(unit_i)
                                    search_outcome.append(unit)
                                unit_i += 1
                        if any(search_outcome):
                            first_col = sheet.col_values(0)
                            alloy_replace = Dictionary(self.C_path).table_alloy_to_replace
                            for alloy_model,replace in alloy_replace.items():
                                alloy_part = re.findall(alloy_model,str(topic))
                                for alloy in alloy_part:
                                    find_part = re.findall(replace[0],str(alloy))
                                    alloy_out = alloy.replace(find_part[0],replace[1])
                                    topic = topic.replace(alloy,alloy_out)
                            alloy_common_type = Dictionary(self.C_path).alloy_writing_type
                            alloy_blank_type = Dictionary(self.C_path).alloy_blank_type
                            outcome_name = []
                            topic_tokenize = nltk.word_tokenize(topic)
                            for word in topic_tokenize:
                                for pattern_1 in alloy_common_type:
                                    outcome_common = re.findall(pattern_1,str(word))
                                    if outcome_common:
                                        outcome_name.append(word)
                                        break        
                            for pattern_2 in alloy_blank_type:
                                outcome_blank = re.findall(pattern_2,str(topic))
                                if outcome_blank:
                                    outcome_name.append(outcome_blank[0])
                                    break
                            fc_ns = []
                            for cell in sheet.col_values(0)[1:]:
                                fc_n = re.findall(self.number_pattern[self.prop_name],str(cell))
                                alphabet_search = re.findall("[A-Za-z]",str(cell))
                                if fc_n and not alphabet_search:
                                    fc_ns.append(cell)
                            len_col = len(sheet.row_values(3))
                            alloy_name_col = None
                            alloy_name_search = []
                            if len_col <= 3:
                                for col_i in range(len_col):
                                    col_info = sheet.col_values(col_i)
                                    if col_i == 0:
                                        col_info = sheet.col_values(col_i)[2:]
                                    if col_info:
                                        for cell in col_info:
                                            for pattern_1 in alloy_common_type:
                                                outcome_common = re.findall(pattern_1,str(cell))
                                                if outcome_common:
                                                    alloy_name_col = col_i
                                                    alloy_name_search.append(col_i)
                                            for pattern_2 in alloy_blank_type:
                                                outcome_blank = re.findall(pattern_2,str(cell))
                                                if outcome_blank:
                                                    alloy_name_col = col_i
                                                    alloy_name_search.append(col_i)
                            else:
                                for col_i in range(3):
                                    col_info = sheet.col_values(col_i)
                                    if col_i == 0:
                                        col_info = sheet.col_values(col_i)[2:]
                                    if col_info:
                                        for cell in col_info:
                                            for pattern_1 in alloy_common_type:
                                                outcome_common = re.findall(pattern_1,str(cell))
                                                if outcome_common:
                                                    alloy_name_col = col_i
                                                    alloy_name_search.append(col_i)
                                            for pattern_2 in alloy_blank_type:
                                                outcome_blank = re.findall(pattern_2,str(cell))
                                                if outcome_blank:
                                                    alloy_name_col = col_i
                                                    alloy_name_search.append(col_i)     
                            if not alloy_name_search:
                                alloy_name_col = 0
                            else:
                                alloy_name_col = alloy_name_search[0]
                            if len(first_col) > 4:
                                for prop_i in range(len(target_prop_row)): 
                                    sub_label = []
                                    curr_col = []
                                    for index_row in range(target_prop_row[prop_i]+1,len(first_col)):
                                        unit_search_parts = []
                                        unit_search_parts.append(topic)
                                        if len(fc_ns) == 0:
                                            name_col = sheet.col_values(alloy_name_col)
                                            material_name = name_col[index_row]
                                            property_single = {}
                                            property_single['table_topic'] = first_col[1]
                                            number = sheet.row_values(index_row)[target_prop_col[prop_i]]
                                            number_inspect = re.findall(self.number_pattern[self.prop_name],str(number))
                                            prop_name = sheet.row_values(target_prop_row[prop_i])[target_prop_col[prop_i]]
                                            unit_search_parts.append(first_col[index_row])
                                            unit_search_parts.append(number)
                                            for unit in sheet.row_values(target_prop_row[0]):
                                                unit_search_parts.append(unit)
                                            for row_s in range(2,target_prop_row[prop_i]+1):
                                                unit_search_parts.append(sheet.row_values(row_s)[target_prop_col[prop_i]])
                                            if number_inspect:
                                                one_info = {}
                                                for prop_index in range(len(sheet.row_values(target_prop_row[prop_i]))):
                                                    prop_name_line = sheet.row_values(target_prop_row[prop_i])[prop_index]
                                                    number_line_line = sheet.row_values(index_row)[prop_index]
                                                    one_info[prop_name_line]=number_line_line
                                                curr_col.append(number)
                                                property_single[prop_name]=number
                                                property_single['other_info']=one_info
                                                property_single['material']=material_name
                                                property_single['doi'] = first_col[0]
                                                if sub_label:
                                                    property_single['child_tag'] = sub_label
                                                for item in unit_search_parts:
                                                    unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                                    if unit_find:
                                                        property_single['unit'] = unit_find[0].replace('degC','°C')
                                                        K_path.append(excel_path)
                                                if 'unit' not in property_single.keys():
                                                    property_single['unit'] = 'no mentioned'   
                                            elif not number_inspect and len(curr_col) != 0:
                                                property_single['material']=material_name
                                                property_single['doi'] = first_col[0]
                                                property_single[prop_name]= number
                                                if sub_label:
                                                    property_single['child_tag'] = sub_label
                                                for item in unit_search_parts:
                                                    unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                                    if unit_find:
                                                        property_single['unit'] = unit_find[0].replace('degC','°C')
                                                        K_path.append(excel_path)
                                                        break
                                                if 'unit' not in property_single.keys():
                                                    property_single['unit'] = 'no mentioned'   
                                            elif not number_inspect and len(curr_col) == 0:
                                                if number and not property_single:
                                                    if number != '-' and number != '--':
                                                        sub_label.append(number)
                                            if property_single:
                                                all_material.append(property_single)
                                        if first_col[index_row] and len(fc_ns) != 0 and len(outcome_name)==1:
                                            material_name = outcome_name[0].replace('~',' ')
                                            property_single = {}
                                            property_single['table_topic'] = first_col[1]
                                            unit_search_parts.append(first_col[index_row])
                                            for row_s in range(2,target_prop_row[prop_i]+1):
                                                unit_search_parts.append(sheet.row_values(row_s)[target_prop_col[prop_i]])
                                            prop_name = sheet.row_values(target_prop_row[prop_i])[target_prop_col[prop_i]]
                                            number = sheet.row_values(index_row)[target_prop_col[prop_i]]
                                            number_inspect = re.findall(self.number_pattern[self.prop_name],str(number))
                                            unit_search_parts.append(number)
                                            if number_inspect:
                                                property_single[prop_name]=number
                                                property_single['material']=material_name
                                                property_single['doi'] = first_col[0]
                                                for item in unit_search_parts:
                                                    unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                                    if unit_find:
                                                        property_single['unit'] = unit_find[0].replace('degC','°C')
                                                        K_path.append(excel_path)
                                                        break
                                                if 'unit' not in property_single.keys():
                                                    property_single['unit'] = 'no mentioned'   
                                            elif not number_inspect and len(curr_col) != 0:
                                                property_single['material']=material_name
                                                property_single['doi'] = first_col[0]
                                                property_single[prop_name]=number
                                                if sub_label:
                                                    property_single['child_tag'] = sub_label
                                                for item in unit_search_parts:
                                                    unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                                    if unit_find:
                                                        property_single['unit'] = unit_find[0].replace('degC','°C')
                                                        K_path.append(excel_path)
                                                        break
                                                if 'unit' not in property_single.keys():
                                                    property_single['unit'] = 'no mentioned'                                                  
                                            elif not number_inspect and len(curr_col) == 0:
                                                if number and not property_single:
                                                    sub_label.append(number)
                                            if property_single:
                                                all_material.append(property_single)
                            else:
                                unit_search_parts = []
                                property_single = {}
                                property_single['table_topic'] = first_col[1]
                                alloy_replace = Dictionary(self.C_path).table_alloy_to_replace
                                for alloy_model,replace in alloy_replace.items():
                                    alloy_part = re.findall(alloy_model,str(topic))
                                    for alloy in alloy_part:
                                        find_part = re.findall(replace[0],str(alloy))
                                        alloy_out = alloy.replace(find_part[0],replace[1])
                                        topic = topic.replace(alloy,alloy_out)
                                alloy_common_type = Dictionary(self.C_path).alloy_writing_type
                                alloy_blank_type = Dictionary(self.C_path).alloy_blank_type
                                outcome_name = []
                                topic_tokenize = nltk.word_tokenize(topic)
                                for word in topic_tokenize:
                                    for pattern_1 in alloy_common_type:
                                        outcome_common = re.findall(pattern_1,str(word))
                                        if outcome_common:
                                            outcome_name.append(word)
                                            break        
                                for pattern_2 in alloy_blank_type:
                                    outcome_blank = re.findall(pattern_2,str(topic))
                                    if outcome_blank and outcome_blank[0] not in outcome_name:
                                        outcome_name.append(outcome_blank[0])
                                        break
                                unit_search_parts.append(first_col[3])
                                unit_search_parts.append(topic)
                                for row_s in range(2,4):
                                    for prop_i in range(len(target_prop_row)):
                                        unit_search_parts.append(sheet.row_values(row_s)[target_prop_col[prop_i]])
                                number_search = re.findall(self.number_pattern[self.prop_name],str(sheet.col_values(0)[2]))
                                if outcome_name and number_search:
                                    for prop_i in range(len(target_prop_row)):
                                        property_single['material'] = outcome_name[0].replace('~',' ')
                                        property_single['doi'] = first_col[0]
                                        number = sheet.row_values(3)[target_prop_col[prop_i]]
                                        unit_search_parts.append(number)
                                        for item in unit_search_parts:
                                            unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                            if unit_find:
                                                property_single['unit'] = unit_find[0].replace('degC','°C')
                                                K_path.append(excel_path)
                                                break
                                        if 'unit' not in property_single.keys():
                                            property_single['unit'] = 'no mentioned'
                                        prop_name = sheet.row_values(target_prop_row[prop_i])[target_prop_col[prop_i]]
                                        property_single[prop_name]=number
                                        all_material.append(property_single)
                                elif not outcome_name and not number_search:
                                    for prop_i in range(len(target_prop_row)):
                                        property_single[sheet.col_values(2)[0]] = first_col[3]
                                        property_single['doi'] = first_col[0]
                                        number = sheet.row_values(3)[target_prop_col[prop_i]]
                                        unit_search_parts.append(number)
                                        for item in unit_search_parts:
                                            unit_find = re.findall(self.unit_pattern[self.prop_name],str(item))
                                            if unit_find:
                                                property_single['unit'] = unit_find[0].replace('degC','°C')
                                                K_path.append(excel_path)
                                                break
                                        if 'unit' not in property_single.keys():
                                            property_single['unit'] = 'no mentioned'
                                        prop_name = sheet.row_values(target_prop_row[prop_i])[target_prop_col[prop_i]]
                                        property_single[prop_name]=number
                                        all_material.append(property_single)  
                                elif not outcome_name and number_search:
                                    for prop_i in range(len(target_prop_row)):
                                        property_single['material'] = 'no mentioned'
                                        property_single['doi'] = first_col[0]
                                        number = sheet.row_values(3)[target_prop_col[prop_i]]
                                        unit_search_parts.append(number)
                                        for item in unit_search_parts:
                                            unit_find = re.findall(self.unit_pattern,str(item))
                                            if unit_find:
                                                property_single['unit'] = unit_find[0].replace('degC','°C')
                                                K_path.append(excel_path)
                                                break
                                        if 'unit' not in property_single.keys():
                                            property_single['unit'] = 'no mentioned'
                                        prop_name = sheet.row_values(target_prop_row[prop_i])[target_prop_col[prop_i]]
                                        property_single[prop_name]=number
                                        all_material.append(property_single)
                   except Exception as e:
                       self.log_wp.print_log("An error in file:{}-sheet:{}---{}!", excel_path, sheet_i, e)
                if all_material:
                    number_prop += 1
                    property_all[excel_path] = all_material
           except Exception as e:
               self.log_wp.print_log("can't open %s ", excel_path)
               self.log_wp.print_log(str(e))
               self.log_wp.print_log("--"*25)
        return property_all


class Get_targetinfo():
    def __init__(self,all_info,out_path):
        self.related_allinfo = all_info
        self.out_path = out_path
        self.log_wp = Log_wp()

    def structure_ele(self):
        xls = openpyxl.Workbook()
        sht = xls.create_sheet(index=0)
        sht.cell(1,1,"File_name")
        sht.cell(1,2,"DOIs")
        sht.cell(1,3,"Material")
        sht.cell(1,4,"Percentage")
        sht.cell(1,5,"Element_info and other_info")
        start_row = 2
        for file,ele in self.related_allinfo.items():
            for material in ele:
                sht.cell(start_row,1,file)
                if 'doi' in material.keys():
                    sht.cell(start_row,2,material['doi'])
                    material.pop('doi')
                if 'material' in material.keys():
                    material_name = material['material']
                    noisy = re.findall('\s*\[.+\]',str(material_name))
                    if noisy:
                        for puc in noisy:
                            material_name = str(material_name).replace(puc,'')
                    sht.cell(start_row,3,material_name)
                    material.pop('material')
                if 'percentage' in material.keys():
                    sht.cell(start_row,4,material['percentage'])
                    material.pop('percentage')
                if material:
                    sht.cell(start_row,5,str(material))
                start_row += 1
        self.log_wp.excel_save(xls, self.out_path)
    
    def structure_prop(self,prop_pattern,prop_name_s):
        xls = openpyxl.Workbook()
        sht = xls.create_sheet(0)
        sht.cell(1,1,"File_name")
        sht.cell(1,2,"DOIs")
        sht.cell(1,3,"Table_topic")
        sht.cell(1,4,"Material")
        sht.cell(1,5,"unit")
        sht.cell(1,6,"Property_name")
        sht.cell(1,7,"Property_value")
        sht.cell(1,8,"Child_tag")
        sht.cell(1,9,"Other_info")
        start_row = 2
        for file,ele in self.related_allinfo.items():
            for material in ele:
                get_prop = None
                sht.cell(start_row,1,file)
                if 'doi' in material.keys():
                    sht.cell(start_row,2,material['doi'])
                    material.pop('doi')
                if 'table_topic' in material.keys():
                    sht.cell(start_row, 3, material['table_topic'])
                    material.pop('table_topic')
                if 'material' in material.keys():
                    material_name = material['material']
                    noisy = re.findall('\s*\[.+\]',str(material_name))
                    if noisy:
                        for puc in noisy:
                            material_name = str(material_name).replace(puc,'')
                    sht.cell(start_row,4,material_name)
                    material.pop('material')
                if 'unit' in material.keys():
                    unit_replace = material['unit'].replace('degC','°C')
                    sht.cell(start_row,5,unit_replace)
                    material.pop('unit')
                if 'child_tag' in material.keys():
                    sht.cell(start_row,8,str(material['child_tag']))
                    material.pop('child_tag')
                if 'other_info' in material.keys():
                    sht.cell(start_row,9,str(material['other_info']))
                    material.pop('other_info')
                if len(material) == 1:
                    for prop_name,value in material.items():
                        sht.cell(start_row,6,str(prop_name))
                        sht.cell(start_row,7,str(value))
                elif len(material) >= 1:
                    for prop_name,value in material.items():
                        for pattern in prop_pattern[prop_name_s]:
                            prop_search = re.findall(pattern,str(prop_name))
                            if prop_search:
                                sht.cell(start_row,6,str(prop_name))
                                sht.cell(start_row,7,str(value))
                                get_prop = True
                                break
                        if get_prop:
                            break
                start_row += 1
        self.log_wp.excel_save(xls, self.out_path)

def transform_comp_outcome(all_composition,ele_list):
    gather_outcome = []
    for file_name,t_info in all_composition.items():
        sole_info = {}
        all_triple_info = []
        for sole_m_info in t_info:
            triple_info = {}
            sole_material = ''
            sole_doi = sole_m_info['doi']
            sole_m_info.pop('doi')
            if 'material' in sole_m_info.keys():
                sole_material = sole_m_info['material']
                noisy = re.findall('\s*\[.+\]',str(sole_material))
                if noisy:
                    for puc in noisy:
                        sole_material = str(sole_material).replace(puc,'')
                triple_info['material'] = sole_material
                sole_m_info.pop('material')            
            for element in ele_list:
                if element in sole_m_info.keys():
                    triple_info[element] = sole_m_info[element]
                    sole_m_info.pop(element)
            if sole_m_info:
                triple_info["other_eleinfo"] = sole_m_info
            all_triple_info.append(triple_info)
        sole_info[sole_doi] = all_triple_info
        gather_outcome.append(sole_info)
        
    return gather_outcome


# if __name__ == '__main__':
#     excels_path = r'C:\Users\win\Desktop\out_table_2'
#     C_path = r'dictionary.ini'
#     te = Table_extraction(excels_path, C_path, prop_name=target_property)
#     # all_composition = te.composition_triple_extraction()
#     info_all = te.property_info_extraction()
#     Get_tar = Get_targetinfo(info_all,r'C:\Users\win\Desktop\train.xlsx')
#     # Get_tar.structure_prop(prop_pattern,target_property)
# #    Get_tar = Get_targetinfo(info_all,r'C:\Users\win\Desktop\prop_allcomposition.xlsx')
#     Get_tar.structure_prop(prop_pattern,target_property)
# #     Get_tar.structure_ele()





__all__ = ["dictionary", "file_io","get_all_attributes","get_full_text","log_wp","output_modified_triple","Phrase_parse","pre_processor","Relation_extraciton","sentence_positioner","T_pre_processor","table","table_extractor_modified","text_with_table"]



# -*- coding: utf-8 -*-
"""
Created on Wed Jan 20 14:49:42 2021

@author: win
"""
import time
import requests
import urllib.request
import pandas as pd
from bs4 import BeautifulSoup
import os
from .log_wp import Log_wp
from lxml import etree

class Get_tinfo_from_html():
    def __init__(self, html_path, output_path):
        self.html_path = html_path
        self.output_path = output_path
        self.log_wp = Log_wp()

    def get_all_url(self,url):
        '''
        return all url on the page
        :param url:url of one page
        :return: all url as list
        '''
        import urllib.request
        from bs4 import BeautifulSoup
        # Masquerading as browser access
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0)\
                    Gecko/20100101 Firefox/23.0'}
        req = urllib.request.Request(url=url, headers=headers)
        html = urllib.request.urlopen(req).read().decode("utf-8")
        # html = urllib.request.urlopen(url).read().decode("utf-8")
        soup = BeautifulSoup(html, features='html.parser')
        tags = soup.find_all('a')
        all_url = []
        for tag in tags:
            all_url.append(str(tag.get('href')).strip())

        return all_url

    def get_table_url(self,doi_info):
        '''
        return table url on the page for Springer and Nature Publishing Group
        :param url:doi_info of article
        :return: table url as list
        '''
        all_url = self.get_all_url(doi_info[0])
        table_url = []
        for i in all_url:
            if "table" in i:
                if 'article' in i:
                    self.log_wp.print_log(str(i))
                    if doi_info[1] in "Springer":
                        table_url.append('https://link.springer.com' + i)
                    else:
                        table_url.append('https://www.nature.com' + i)
        if len(table_url) == 0:
            self.log_wp.print_log("There is no table url in this article!")
        self.log_wp.print_log(str(table_url))

        return table_url

    def doi_info(self,doi_str):
        '''
        get url and database of doi
        :param doi_str: doi as str
        :return: doi_info=[doi_url,doi_database]
        '''
        global doi_url
        doi_info = []
        if doi_str[0:7] in "10.1016":
            doi_url = "https://doi.org/" + doi_str
            doi_database = "Elsevier"
        elif doi_str[0:7] in ["10.1007", "10.1361", "10.1023"]:
            doi_url = "https://link.springer.com/article/" + doi_str
            doi_database = "Springer"
        elif doi_str[0:7] in "10.1080":
            doi_url = "https://doi.org/" + doi_str
            doi_database = "Taylor & Francis Online"
        elif doi_str[0:7] in ["10.1002", "10.1111"]:
            doi_url = "https://onlinelibrary.wiley.com/doi/" + doi_str
            doi_database = "Wiley Blackwell"
        elif doi_str[0:7] in "10.1115":
            doi_url = "https://doi.org/" + doi_str
            doi_database = "ASME International"
        elif doi_str[0:7] in "10.3390":
            # 解决MDPI页面跳转
            all_url = self.get_all_url("https://doi.org/" + doi_str)
            for url_str in all_url:
                if "htm" in url_str:
                    doi_url = "https://www.mdpi.com/" + url_str
                    break
            doi_database = "MDPI"
        elif doi_str[0:7] == "10.1038":
            doi_url = "https://doi.org/" + doi_str
            doi_database = "Nature Publishing Group"
        else:
            doi_url = "other URL"
            doi_database = "other database"
        doi_info.append(doi_url)
        doi_info.append(doi_database)
        doi_info.append(doi_str)

        return doi_info

    def fileName(self,name):
        st = '\|/:?*<>;'
        for s in st:
            if s in name:
                name = name.replace(s, '-')

        return name

    def get_table(self,doi_info,path=r'table.xlsx'):
        '''
        get all table name from the page,
        :param doi_info: [doi_url,1doi_info_name]str
        :param rt: requests.get(url).text
        :return:
        '''
        table_name = []
        if doi_info[1] in ['Springer', 'Nature Publishing Group']:
            table_url = self.get_table_url(doi_info)

            if len(table_url) != 0:

                with pd.ExcelWriter(path) as writer:
                    # 保存并写入多个sheet
                    for p in range(len(table_url)):
                        time.sleep(1)
                        self.log_wp.print_log("Start crawling the page")
                        r = requests.get(table_url[p])
                        rt = r.text
                        # 将html转为纯文本, header=None, index_col=None
                        try:
                            df = pd.read_html(rt)
                            self.log_wp.print_log("complete!")
                        except:
                            self.log_wp.print_log('format of table ' + str(p) + ' is PDF' )
                            continue
                        # 解析表格title
                        start = rt.find("<h1")
                        end = rt.rfind("</h1>")

                        title_str = ''
                        for i in range(start, end + 5):
                            title_str += rt[i]
                        title_start = title_str.find("Table")
                        title_end = title_str.find("</h1>")
                        title = ''
                        for j in range(title_start, title_end):
                            self.log_wp.print_log(str(title_str[j]))
                            title += title_str[j]
                        table_name.append(title)
                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 将文章的doi加入其中
                        row_doi = [doi_info[2]]
                        for j in range(len(df[0].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)  # 将title加入表格其中
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(title)
                        for j in range(len(df[0].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[0].columns))
                        for i in range(len(df[0])):
                            # te.append()
                            table_te.append(list(df[0].iloc[i]))
                        df[0] = pd.DataFrame(data=table_te)
                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        df[0].to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this page: " + self.fileName(doi_info[0]))

        elif doi_info[1] in 'Taylor & Francis Online':

            rt = self.get_rt(doi_info[0])
            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 寻找页面构造成table_name的list
            table_name = []
            for page1 in page.find_all('b'):
                name = page1.text
                if 'Table' in name:
                    table_name.append(name)
            # 删除重复表格
            del table_name[int(len(table_name) / 2):len(table_name)]

            # 删除表格中重复的title
            count = 0
            for t in table_name:
                if 'Table 1' in t:
                    count += 1
            if count > 1:
                del table_name[1:(len(table_name)):2]

            if len(table_name) != 0:
                with pd.ExcelWriter(path) as writer:
                    for p in range(len(table_name)):
                        df = pd.read_html(rt)
                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        df = pd.DataFrame(data=table_te)
                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        df.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this page: " + self.fileName(doi_info[0]))

        elif doi_info[1] in 'MDPI':

            rt = self.get_rt(doi_info[0])
            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            ####找寻表格的title
            table_name = []
            for page1 in page.find_all('caption'):
                name = page1.text
                name = name.replace('\n', '')  # 清除title中的冗余字段
                table_name.append(name)
            self.log_wp.print_log(str(table_name))
            if len(table_name) != 0:
                with pd.ExcelWriter(path) as writer:
                    # 爬取HTML内容
                    time.sleep(1)
                    self.log_wp.print_log("Start crawling the page")
                    r = requests.get(doi_info[0])
                    rt = r.text
                    # 将html转为纯文本, header=None, index_col=None
                    df = pd.read_html(rt)
                    self.log_wp.print_log("complete!")

                    for p in range(len(table_name)):
                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        fa = pd.DataFrame(data=table_te)
                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this page: " + self.fileName(doi_info[0]))
        elif doi_info[1] in "ASME International":
            import re
            rt = self.get_rt(doi_info[0])
            # rt = re.sub(r'<.*?>', lambda g: g.group(0).upper(), rt)

            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 将html转为纯文本, header=None, index_col=None
            ####找寻表格的title
            table_name = []
            for page1 in page.find_all('div'):
                name = page1.text
                if 'Table' in name[0:5]:
                    if ' ' in name[-1]:
                        table_name.append(name)
            if len(table_name) != 0:
                df = pd.read_html(rt)
                self.log_wp.print_log("complete!")
                del df[0:len(df):2]
                with pd.ExcelWriter(path) as writer:
                    # 爬取HTML内容
                    for p in range(len(df)):

                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        fa = pd.DataFrame(data=table_te)

                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this page: " + doi_info[0])
        elif doi_info[1] in "Wiley Blackwell":

            rt = self.get_rt(doi_info[0])
            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 将html转为纯文本, header=None, index_col=None
            table_name = []
            for page1 in page.find_all('header'):
                name = page1.text
                if 'Table' in name:
                    name = ' '.join(name.split())
                    table_name.append(name.replace('\n', ''))
            try:
                if len(table_name) != 0:
                    df = pd.read_html(rt)
                    with pd.ExcelWriter(path) as writer:
                        # 爬取HTML内容
                        for p in range(len(df)):
                            # 读取table的title并写入dataframe首行
                            table_te = []
                            # 读取文章的doi并写入表格第一行
                            row_doi = [doi_info[2]]
                            for j in range(len(df[p].columns) - 1):
                                row_doi.append('')
                            table_te.append(row_doi)
                            # 将title加入表格其中
                            row_title = []
                            row_title.append(table_name[p])
                            for j in range(len(df[p].columns) - 1):
                                row_title.append('')
                            table_te.append(row_title)
                            table_te.append(list(df[p].columns))
                            for i in range(len(df[p])):
                                # te.append()
                                table_te.append(list(df[p].iloc[i]))
                            fa = pd.DataFrame(data=table_te)

                            # 写入excel
                            sheet_name = 'table' + str(p + 1)
                            # 写入excel时不加index和columns
                            fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
                else:
                    self.log_wp.print_log(" Cannot find table in this page: " + doi_info[0])
            except:
                pself.log_wp.print_log(" Cannot find table in this page: " + doi_info[0])


        else:
            self.log_wp.print_log("Please try other function!")

        return table_name

    def get_rt(self,url):
        '''

        :param url:
        :return:
        '''
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/20100101 Firefox/23.0'}
        time.sleep(1)
        self.log_wp.print_log("Start crawling the page")
        r = requests.get(url, headers=headers)
        r.encoding = 'utf-8'
        rt = r.text
        self.log_wp.print_log("complete!")

        return rt

    ############## 导入excel文件 ###########################
    def load_doi(self,path):
        import xlrd
        data = xlrd.open_workbook(path)
        table = data.sheet_by_index(0)
        # 获取表格所有行数
        nrows = table.nrows
        doi_li = []
        # excel文件中的doi处于第一列且省略表头
        for row in range(nrows):
            table_doi = table.row_values(row, start_colx=0, end_colx=None)[0]
            doi_li.append(table_doi)

        return doi_li

    ########################################################
    def getHtml(self,url):
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/20100101 Firefox/23.0'}
        req = urllib.request.Request(url=url, headers=headers)

        html = urllib.request.urlopen(req).read()

        return html

    def saveHtml(self,file_name, file_content):
        # 注意windows文件命名的禁用符，比如 /
        # path = 'C:/Users/T_sha/Desktop/DOI_MDPI/'
        # with open(file_name.replace('/', '_') + path+".html", "wb") as f:
        with open(file_name + ".html", "wb") as f:
            # 写文件用bytes而不是str，所以要转码
            f.write(file_content)

    def down_html(self,doi_li, path=''):
        for s in range(len(doi_li)):
            name = self.fileName(doi_li[s])
            url_te = self.doi_info(doi_li[s])[0]
            html = self.getHtml(url_te)
            self.saveHtml(path + name, html)
            self.log_wp.print_log('html_' + str(s + 1) + " download completed!")

    def doi_renamed(self,html_name):
        #删除‘.html’后缀
        name = html_name[0:-5]

        if len(html_name) > 7:
            #将doi中的'-'转为'/'
            name = self.str_sub(name,7,'/')
        else:
            self.log_wp.print_log('Your file name is wrong!')


        return name

    #string指定p位置替换c
    def str_sub(self,string, p, c):
        new = []
        for s in string:
            new.append(s)
        new[p] = c
        return ''.join(new)

    def load_html(self,html_input_path):
        #输入导入html路径
        html_name = self.doi_renamed(os.path.basename(html_input_path))
        f = open(html_input_path,"r",encoding="utf-8") #读取文件
        ft = f.read()#把文件内容转化为字符串
        return ft,html_name

    def get_table_html(self,doi_info,rt,path=r'table.xlsx'):
        '''
        get all table name from the page,
        :param doi_info: [doi_url,1doi_info_name]str
        :param rt: requests.get(url).text
        :return:
        '''
        if doi_info[1] in 'Taylor & Francis Online':

            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 寻找页面构造成table_name的list
            table_name = []
            for page1 in page.find_all('b'):
                name = page1.text
                if 'Table' in name:
                    table_name.append(name)
            # 删除重复表格
            del table_name[int(len(table_name) / 2):len(table_name)]

            # 删除表格中重复的title
            count = 0
            for t in table_name:
                if 'Table 1' in t:
                    count += 1
            if count > 1:
                del table_name[1:(len(table_name)):2]

            if len(table_name) != 0:
                with pd.ExcelWriter(path) as writer:
                    for p in range(len(table_name)):
                        df = pd.read_html(rt)
                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        df = pd.DataFrame(data=table_te)
                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        df.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this file: " + self.fileName(doi_info[2]) + '.html')

        elif doi_info[1] in 'MDPI':

            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            ####找寻表格的title
            table_name = []
            for page1 in page.find_all('caption'):
                name = page1.text
                name = name.replace('\n', '')  # 清除title中的冗余字段
                table_name.append(name)
            self.log_wp.print_log(str(table_name))
            if len(table_name) != 0:
                with pd.ExcelWriter(path) as writer:
                    df = pd.read_html(rt)
                    for p in range(len(table_name)):
                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        fa = pd.DataFrame(data=table_te)
                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this file: " + self.fileName(doi_info[2]) + '.html')
        elif doi_info[1] in "ASME International":
            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 将html转为纯文本, header=None, index_col=None
            ####找寻表格的title
            table_name = []
            for page1 in page.find_all('div'):
                name = page1.text
                if 'Table' in name[0:5]:
                    if ' ' in name[-1]:
                        table_name.append(name)
            if len(table_name) != 0:
                df = pd.read_html(rt)
                self.log_wp.print_log("complete!")
                del df[0:len(df):2]
                with pd.ExcelWriter(path) as writer:
                    # 爬取HTML内容
                    for p in range(len(df)):

                        # 读取table的title并写入dataframe首行
                        table_te = []
                        # 读取文章的doi并写入表格第一行
                        row_doi = [doi_info[2]]
                        for j in range(len(df[p].columns) - 1):
                            row_doi.append('')
                        table_te.append(row_doi)
                        # 将title加入表格其中
                        row_title = []
                        row_title.append(table_name[p])
                        for j in range(len(df[p].columns) - 1):
                            row_title.append('')
                        table_te.append(row_title)
                        table_te.append(list(df[p].columns))
                        for i in range(len(df[p])):
                            # te.append()
                            table_te.append(list(df[p].iloc[i]))
                        fa = pd.DataFrame(data=table_te)

                        # 写入excel
                        sheet_name = 'table' + str(p + 1)
                        # 写入excel时不加index和columns
                        fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
            else:
                self.log_wp.print_log(" Cannot find table in this file: " + self.fileName(doi_info[2]) + '.html')
        elif doi_info[1] in ["Springer", "Nature Publishing Group","Wiley Blackwell"]:
            page = BeautifulSoup(rt, 'lxml')  # 利用BeautifulSoup取得网页代码
            # 将html转为纯文本, header=None, index_col=None
            table_name = []
            for page1 in page.find_all('header'):
                name = page1.text
                if 'Table' in name:
                    name = ' '.join(name.split())
                    table_name.append(name.replace('\n', ''))
            try:
                if len(table_name) != 0:
                    df = pd.read_html(rt)
                    with pd.ExcelWriter(path) as writer:
                        # 爬取HTML内容
                        for p in range(len(df)):
                            # 读取table的title并写入dataframe首行
                            table_te = []
                            # 读取文章的doi并写入表格第一行
                            row_doi = [doi_info[2]]
                            for j in range(len(df[p].columns) - 1):
                                row_doi.append('')
                            table_te.append(row_doi)
                            # 将title加入表格其中
                            row_title = []
                            row_title.append(table_name[p])
                            for j in range(len(df[p].columns) - 1):
                                row_title.append('')
                            table_te.append(row_title)
                            table_te.append(list(df[p].columns))
                            for i in range(len(df[p])):
                                # te.append()
                                table_te.append(list(df[p].iloc[i]))
                            fa = pd.DataFrame(data=table_te)

                            # 写入excel
                            sheet_name = 'table' + str(p + 1)
                            # 写入excel时不加index和columns
                            fa.to_excel(writer, sheet_name=sheet_name, header=None, index=False)
                else:
                    self.log_wp.print_log(" Cannot find table in this file: " + self.fileName(doi_info[2]) + '.html')
            except:
                self.log_wp.print_log(" Cannot find table in this file: " + self.fileName(doi_info[2]) + '.html')
        else:
            table_name = []
            self.log_wp.print_log("This doi belongs to other databases!")

        return table_name

    def run(self):
        html_list = os.listdir(self.html_path)
        for html_file in html_list:
            path = self.html_path + '/' + html_file
            rt, html_name = self.load_html(path)
            ExcelName = self.fileName(html_name)
            output_path = self.output_path + '/' + '%s.xlsx'
            TableName = self.get_table_html(self.doi_info(html_name), rt, output_path % (ExcelName))

##############本地html文件表格抽取#########################
'''
用于测试通过本地HTML文档表格抽取效果
taylor(table)    10.1080-02670836.1987.11782270
ASME(table) 10.1115-1.1447238   10.1115-1.3225007
wiley(table) 10.1002-adem.201200055  10.1002-1527-2648(200
MDPI(table) 10.3390-ma8095299   10.3390-met4010001
需要进行HTML文档合并
NPG(table) 10.1038-s41529-018-0046-1.mhtml  10.1038-s41467-020-14820-0
springer(table)10.1007-s11837-014-1181-y
'''


# -*- coding: utf-8 -*-
"""
Created on Wed Jun 10 15:06:30 2020

@author: 35732
"""
import xlrd
import xlwt
import os
from .pre_processor import Pre_processor
from .log_wp import Log_wp

class All_attributes:
    def __init__(self,prop_name,txt_name,text_path,triple_path,out_path,C_path):
        self.txt_name = txt_name
        self.text_path = text_path
        self.triple_path = triple_path
        self.prop_name = prop_name
        self.out_path = out_path
        self.C_path = C_path
        self.log_wp = Log_wp()

    def get_toexcel(self):
        xls = xlwt.Workbook()
        sheet = xls.add_sheet("all_attributes")
        sheet.write(0,0,'txt_name')
        sheet.write(0,1,'Full_text')
        sheet.write(0,2,'Target_sentences')
        sheet.write(0,3,'alloy')
        sheet.write(0,4,'property')
        sheet.write(0,5,'value')

        #triples_tuple
        data_triples = xlrd.open_workbook(self.triple_path)
        table_triples = data_triples.sheet_by_index(0)
        for x in range(0,len(table_triples.col_values(0))):
            sheet.write(x+1,1,str(table_triples.col_values(0)[x]))
            sheet.write(x+1,2,str(table_triples.col_values(1)[x]))  
            sheet.write(x+1,3,str(table_triples.col_values(2)[x]))  
            sheet.write(x+1,4,str(table_triples.col_values(3)[x]))  
            sheet.write(x+1,5,str(table_triples.col_values(4)[x]))  
        
        #txt_name
        k = 0
        for i in range(0,len(table_triples.col_values(0))):
            if table_triples.col_values(0)[i]:
                sheet.write(i+1,0,str(self.txt_name[k]))
                k += 1
        #text + filtered_text_1 + Target sentences
        txt_name = os.listdir(self.text_path) 
        process_text = []
        for i in range(0,len(txt_name)):
            file = open(self.text_path + '/'+str(i)+'.txt' ,'r',encoding='utf-8')
            sole_text = file.read()
            pre_processor = Pre_processor(sole_text,self.C_path)
            filter_txt = pre_processor.pre_processor()
            path_2 = self.text_path +'/proprecess'
            isExists =os.path.exists(path_2)
            if isExists:
                text_path = path_2 + '/'+str(i)+'.txt' 
                file = open(text_path ,'w',encoding='utf-8')
                file.write(filter_txt)
                process_text.append(text_path)
            else:
                os.makedirs(path_2)
                text_path = path_2 + '/'+str(i)+'.txt' 
                file = open(text_path ,'w',encoding='utf-8')
                file.write(filter_txt)
                process_text.append(text_path)
        self.log_wp.excel_save(xls, self.out_path)


# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 10:32:00 2019

@author: JiangXue
"""


from pandas import DataFrame

def data_archive(sample,filename,sheet):   
    data  = DataFrame(sample)
    DataFrame(data).to_excel(filename,sheet_name=sheet)
    

if __name__ == '__main__':
    sample = [[1,2],[1,3],[3,4]]
    data_archive(sample,"1.xlsx","test")


from bson.objectid import ObjectId
from copy import deepcopy
import pandas as pd
import numpy as np


class Document(dict):

    structure = dict()

    def __init__(self, initializer=None):
        for key, (struct, value) in self.structure.items():
            dict.__setitem__(self, key, deepcopy(value))

        if initializer is not None:
            for key, value in initializer.items():
                self.__setitem__(self, key, deepcopy(value))

    def __setitem__(self, key, value):
        if key not in self.structure:
            raise KeyError("Invalid key used: '" + key + "'.")

        expected = self.structure[key][0]

        if value is not None:
            if type(expected) == list:
                if type(value) != list:
                    raise TypeError("Invalid type used: Expected '[" + 
                    self.structure[key][0][0].__name__ + "]' but key '" + key + "' is not array.")
                if not all(isinstance(x, expected[0]) for x in value):
                    raise TypeError("Invalid type used: Expected '[" + 
                    self.structure[key][0][0].__name__ + "]' but got '" + type(value).__name__ 
                    + "' for item in key '" + key + "'.")
            elif not isinstance(value, expected):
                raise TypeError("Invalid type used: Expected '" + 
                self.structure[key][0].__name__ + "' but got '" + 
                type(value).__name__ + "' for key '" + key + "'.")

        return dict.__setitem__(self, key, value)

   
class Link(Document):
    structure = dict({
        'name':(str, None),
        'link_ref':(list, [])
    })

class Attribute(Document):
    structure = dict({
        'name':(str, None), 
        'value':(float, 0),
        'links': ([Link], []),
        'string_value' : (str, None),
        'unit': (str, None),
        'value_ref':(list, []),
        'attr_ref':(list, [])
    })

# top level of graph structure
class Entity(Document):
    structure = dict({
        'name':(str, None),
        'attributes':([Attribute], []),
        'links':([Link], []),
        'descriptor':(str, None),
        'ent_ref':(list, [])
    })

class Table(Document):
    structure = dict({
        '_id': (ObjectId, None),
        'paper_doi':(str, None),
        'order':(int, 0),
        'act_table':(list, None), 
        'entities':([Entity], []), 
        'caption':(str, None),
        'caption_ref':(list, []),
        'composition_table':(bool, False), 
        'footer':(dict, None)
    })











# -*- coding: utf-8 -*-
"""
Created on Mon May 18 08:55:54 2020

@author: 35732
"""
#triple recognition-Pre_processor
import re
import nltk
from .dictionary import Dictionary

class T_pre_processor:
    def __init__(self,all_text,prop_name,C_path):
        self.C_path = C_path
        self.all_text = all_text
        self.dict_info = Dictionary(self.C_path)
        self.prop_name = prop_name
        
    def processor(self):
        all_txt = self.all_text
        replace_word = self.dict_info.replace_word
        unit_replace = self.dict_info.unit_replace
        paras = self.dict_info.paras_to_replace
        alloy_replace = self.dict_info.alloy_to_replace
        writing_type = self.dict_info.prop_writing_type
        for element in writing_type[self.prop_name]:
            all_txt = all_txt.replace(element,self.prop_name)
        for old_word,new_word in replace_word.items():
            all_txt = all_txt.replace(old_word,new_word)
        for para_model,change_place in paras.items():
            paras_all = re.findall(para_model,all_txt)
            for para in paras_all:
                find_word = re.findall(change_place[0],para)
                para_out = para.replace(find_word[0],change_place[1])
                all_txt = all_txt.replace(para,para_out)
        for alloy_model,replace in alloy_replace.items():
            alloy_part = re.findall(alloy_model,all_txt)
            for alloy in alloy_part:
                find_part = re.findall(replace[0],alloy)
                alloy_out = alloy.replace(find_part[0],replace[1])
                all_txt = all_txt.replace(alloy,alloy_out)
        sent_list = nltk.sent_tokenize(all_txt)
        patterns = self.dict_info.no_unit_para
        alloy_blank_type = self.dict_info.alloy_blank_type
        for pattern in alloy_blank_type:
            outcome = re.findall(pattern,all_txt)
        para = patterns[self.prop_name]
        for sent in sent_list:
            word_list = nltk.word_tokenize(sent)
            list_order = list(set(word_list))
            for word in list_order:
                if word not in outcome:
                    for pattern in para:
                        number = re.findall(pattern,word)
                        if unit_replace[self.prop_name][0] in sent and number:
                            all_txt = all_txt.replace(word,word+unit_replace[self.prop_name][0])
                        if unit_replace[self.prop_name][0] not in sent and number:
                            all_txt = all_txt.replace(word,word+unit_replace[self.prop_name][-1])
        return all_txt


# -*- coding: utf-8 -*-
"""
Created on Tue Sep 24 11:38:48 2019

@author: JiangXue
"""

import requests 
import xlrd

def data_totxt(sample,path):
    f = open(path,'w',encoding='utf-8')
    f.write(sample)
    f.close()
    

header={'Accept':'text/plain','CR-TDM-Rate-Limit':'4000','CR-TDM-Rate-Limit-Remaining':'76','CR-TDM-Rate-Limit-Reset':'1378072800'}

url_publisher = "https://api.elsevier.com/content/article/doi/" #如果获取全文的话，将abstract替换成article
APIKey="APIKey=aa1b99025fb64e69ff251ad6de1f784a" #developer Elsevier 申请的
arformat = "text/plain" #text/xml,text/plain
data = xlrd.open_workbook(r'alloyDOI.xlsx')
table =data.sheet_by_index(0)
 #输出为数组的形式，好像只能指定某一行输出
start=4671 #开始的行
end=4972 #结束的行
rows=end-start
list_values=[]
for x in range(start,end):
  values=[]
  row =table.row_values(x)
  values.append(row[1])
  list_values.append(values)
dois = list_values
count = len(dois)
articles = []
k = 4671
for i in range(0,count):
    url = url_publisher+str(dois[i][0])+"?"+APIKey+"&httpAccept="+arformat
    #url = "https://api.elsevier.com/content/article/doi/10.1016/j.matchar.2013.01.008?APIKey=36697d0dea0745f5f236356d7f5cd38f&httpAccept=text/plain"
    r = requests.get(url,headers=header)
    print(r.content)
#    path = str(i)+".txt"
    path = r'E:\文本挖掘\陶瓷膜的文献\文献下载doi\web_4971_txt/'+str(k)+".txt"
    k += 1
    data_totxt(r.content.decode(),path)
    






# -*- coding: utf-8 -*-
"""
Created on Wed Jul 29 09:23:52 2020

@author: 35732
"""

from __future__ import print_function
from setuptools import setup, find_packages
import sys
import setuptools
 
setup(
      name="SuperalloyDigger",
      version="0.0.9",
      author="Weiren_Wang",
      author_email="357329191@qq.com",
      description="Automatic extraction of chemical compositions and properties from the scientific literature of superalloy, covering specific chemical composition, density, solvus temperature, solidus temperature, and liquidus temperature.",
      long_description=open("README.md",encoding="utf-8").read(),
      keywords='text-mining, superalloy, informatics, nlp, txt, science, scientific',
      license="MIT",
      url="https://github.com/Weiren1996/superalloydigger",
      packages=setuptools.find_packages(),
      long_description_content_type="text/markdown",
      classifiers=[
              "Environment :: Web Environment",
              "Intended Audience :: Science/Research",
              "Operating System :: Microsoft :: Windows",
              "Programming Language :: Python :: 3",
              "Topic :: Internet",
              "Topic :: Scientific/Engineering",
              "Topic :: Text Processing",
              ],
      python_requires='>=3.6'
)


# -*- coding: utf-8 -*-
"""
Created on Wed Jun 10 12:04:34 2020

@author: 35732
"""
# 下载纯文本和xml
import requests
import xlrd
import xlwt


class Article_archive_doi:
    def __init__(self, header, filename, url_publisher, APIKey, arformat, path, text_outpath):
        self.filename = filename
        self.url_publisher = url_publisher
        self.APIKey = APIKey
        self.arformat = arformat
        self.path = path
        self.header = header
        self.out_path = text_outpath

    def data_totxt(self, sample, path):
        f = open(path, 'w', encoding='utf-8')
        f.write(sample)
        f.close()

    def httprequest(self):
        xls = xlwt.Workbook()
        sheet = xls.add_sheet("doi-text_path")
        data = xlrd.open_workbook(self.filename)
        table = data.sheet_by_index(0)
        # 输出为数组的形式，好像只能指定某一行输出
        list_values = []
        #        end = table.nrows
        for x in range(1, len(table.col_values(1))):  # 默认从第2行第2列往下进行
            values = []
            row = table.row_values(x)
            values.append(row[1])
            list_values.append(values)
        dois = list_values
        print(dois)
        count = len(dois)
        for i in range(0, count):
            url = self.url_publisher + dois[i][0] + "?" + self.APIKey + "&httpAccept=" + self.arformat
            r = requests.get(url, headers=self.header)
            content = r.content.decode()
            print(self.path)
            path = self.path + '/' + str(i) + '.txt'
            print(path)
            self.data_totxt(content, path)
            # 将输出路径对应写在源文件doi后边
            sheet.write(i, 0, dois[i][0])
            sheet.write(i, 1, path)
        xls.save(self.out_path)



# -*- coding: utf-8 -*-
"""
Created on Mon Apr 27 23:15:37 2020

@author: 35732
"""
# from configparser import ConfigParser
import configparser

class Dictionary:
    def __init__(self, path):
        cp = configparser.RawConfigParser()
        cp.read(path,'UTF-8')
        self.replace_word = eval(cp.get("DICTIONARY",'replace_word'))
        self.alloy_to_replace = eval(cp.get("DICTIONARY",'alloy_to_replace'))
        self.paras_to_replace = eval(cp.get("DICTIONARY",'paras_to_replace'))
        self.alloy_writing_type = eval(cp.get("DICTIONARY",'alloy_writing_type'))
        self.alloy_blank_type = eval(cp.get("DICTIONARY",'alloy_blank_type'))
        self.prop_writing_type = eval(cp.get("DICTIONARY",'prop_writing_type'))
        self.value_wt = eval(cp.get("DICTIONARY",'value_wt'))
        self.other_phase = eval(cp.get("DICTIONARY",'other_phase'))
        self.unit_replace = eval(cp.get("DICTIONARY",'unit_replace'))
        self.no_unit_para = eval(cp.get("DICTIONARY",'no_unit_para'))
        self.other_quality = eval(cp.get("DICTIONARY",'other_quality'))